@@ -4,7 +4,7 @@
  * Copyright (c) 2000-2004 Anton Altaparmakov
  * Copyright (c) 2004-2005 Richard Russon
  * Copyright (c) 2005-2006 Yura Pakhuchiy
- * Copyright (c) 2005-2008 Szabolcs Szakacsits
+ * Copyright (c) 2005-2009 Szabolcs Szakacsits
  *
  * This program/include file is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as published
@@ -52,8 +52,8 @@
 
 #define MS_EXCLUSIVE 0x08000000
 
-#ifndef MS_FORCE
-#define MS_FORCE     0x10000000
+#ifndef MS_RECOVER
+#define MS_RECOVER   0x10000000
 #endif
 
 #define MS_IGNORE_HIBERFILE   0x20000000
@@ -61,11 +61,13 @@
 /* Forward declaration */
 typedef struct _ntfs_volume ntfs_volume;
 
+#include "param.h"
 #include "types.h"
 #include "support.h"
 #include "device.h"
 #include "inode.h"
 #include "attrib.h"
+#include "index.h"
 
 /**
  * enum ntfs_mount_flags -
@@ -154,7 +156,7 @@ struct _ntfs_volume {
 	ntfs_inode *vol_ni;	/* ntfs_inode structure for FILE_Volume. */
 	u8 major_ver;		/* Ntfs major version of volume. */
 	u8 minor_ver;		/* Ntfs minor version of volume. */
-	u16 flags;		/* Bit array of VOLUME_* flags. */
+	le16 flags;		/* Bit array of VOLUME_* flags. */
 
 	u16 sector_size;	/* Byte size of a sector. */
 	u8 sector_size_bits;	/* Log(2) of the byte size of a sector. */
@@ -167,6 +169,7 @@ struct _ntfs_volume {
 
 	/* Variables used by the cluster and mft allocators. */
 	u8 mft_zone_multiplier;	/* Initial mft zone multiplier. */
+	u8 full_zones;		/* cluster zones which are full */
 	s64 mft_data_pos;	/* Mft record number at which to allocate the
 				   next mft record. */
 	LCN mft_zone_start;	/* First cluster of the mft zone. */
@@ -196,6 +199,12 @@ struct _ntfs_volume {
 				   bit means that the mft record is in use and
 				   vice versa. */
 
+	ntfs_inode *secure_ni;	/* ntfs_inode structure for FILE $Secure */
+	ntfs_index_context *secure_xsii; /* index for using $Secure:$SII */
+	ntfs_index_context *secure_xsdh; /* index for using $Secure:$SDH */
+	int secure_reentry;  /* check for non-rentries */
+	unsigned int secure_flags;  /* flags, see security.h for values */
+
 	int mftmirr_size;	/* Size of the FILE_MFTMirr in mft records. */
 	LCN mftmirr_lcn;	/* Logical cluster number of the data attribute
 				   for FILE_MFTMirr. */
@@ -217,13 +226,29 @@ struct _ntfs_volume {
 	s64 free_clusters; 	/* Track the number of free clusters which
 				   greatly improves statfs() performance */
 	s64 free_mft_records; 	/* Same for free mft records (see above) */
+	BOOL efs_raw;		/* volume is mounted for raw access to
+				   efs-encrypted files */
+
+#if CACHE_INODE_SIZE
+	struct CACHE_HEADER *xinode_cache;
+#endif
+#if CACHE_NIDATA_SIZE
+	struct CACHE_HEADER *nidata_cache;
+#endif
+#if CACHE_LOOKUP_SIZE
+	struct CACHE_HEADER *lookup_cache;
+#endif
+#if CACHE_SECURID_SIZE
+	struct CACHE_HEADER *securid_cache;
+#endif
+#if CACHE_LEGACY_SIZE
+	struct CACHE_HEADER *legacy_cache;
+#endif
 
-	/* Temp: for directory handling */
-	void *private_data;	/* ntfs_dir for . */
-	void *private_bmp1;	/* ntfs_bmp for $MFT/$BITMAP */
-	void *private_bmp2;	/* ntfs_bmp for $Bitmap */
 };
 
+extern const char *ntfs_home;
+
 extern ntfs_volume *ntfs_volume_alloc(void);
 
 extern ntfs_volume *ntfs_volume_startup(struct ntfs_device *dev,
@@ -239,9 +264,14 @@ extern int ntfs_version_is_supported(ntfs_volume *vol);
 extern int ntfs_volume_check_hiberfile(ntfs_volume *vol, int verbose);
 extern int ntfs_logfile_reset(ntfs_volume *vol);
 
-extern int ntfs_volume_write_flags(ntfs_volume *vol, const u16 flags);
+extern int ntfs_volume_write_flags(ntfs_volume *vol, const le16 flags);
 
 extern int ntfs_volume_error(int err);
+extern void ntfs_mount_error(const char *vol, const char *mntpoint, int err);
+
+extern int ntfs_volume_get_free_space(ntfs_volume *vol);
+
+extern int ntfs_set_locale(void);
 
 #endif /* defined _NTFS_VOLUME_H */
 
