@@ -439,7 +439,7 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
     const char *mnt = *mntp;
     const char *origmnt = mnt;
 
-    res = lstat(mnt, stbuf);
+    res = stat(mnt, stbuf);
     if (res == -1) {
         fprintf(stderr, "%s: failed to access mountpoint %s: %s\n",
                 progname, mnt, strerror(errno));
@@ -588,7 +588,8 @@ static int mount_fuse(const char *mnt, const char *opts)
 			&source, &mnt_opts);
 
     if (currdir_fd != -1) {
-        fchdir(currdir_fd);
+	 __attribute__((unused))int ignored_fchdir_status =
+	        fchdir(currdir_fd);
         close(currdir_fd);
     }
     if (mountpoint_fd != -1)
